// antcn.cc
//
// When        Who  What
// $Log: antcn.cc,v $
// Revision 1.2  1997/03/13  12:42:31  prog
// C++ comments in log.
//
// Revision 1.1  1997/03/13  12:29:19  prog
// Initial revision
//
// 1996-06-10  amn  Longer 1.5 sec delay.
// 1996-02-23  amn  Adding 0.3 sec delay before "silent" onsource queries
//                  from VAX vlbi_tracker_p.
// 1995-11-01  amn  Cleaning up extra log messages.
// 1995-10-31  amn  Parallel port I/O.
// 1995-10-29  amn  Initial testing.
// 1995-10-25  amn  Original Metsahovi version.
//
// This was the stub version of antcn (ANTenna CoNtrol program) of fs-9.0.7.

/* Input */
/* IP(1) = mode
       0 = initialize LU
       1 = pointing (from SOURCE command)
       2 = offset (from RADECOFF, AZELOFF, or XYOFF commands)
       3 = on/off source status (from ONSOURCE command)
       4 = direct communications (from ANTENNA command)
       5 = on/off source status for pointing programs
       6 = reserved for future focus control
       7 = log tracking data (from TRACK command)
  8 - 99 = reserved for future use
100 - 32767 = for site specific use

   IP(2) = class number (mode 4 only)
   IP(3) = number of records in class (mode 4 only)
   IP(4) - not used
   IP(5) - not used
*/

/* Output */
/*  IP(1) = class with returned message
      (2) = number of records in class
      (3) = error number
            0 - ok
           -1 - illegal mode
           -2 - timeout
           -3 - wrong number of characters in response
           -4 - interface not set to remote
           -5 - error return from antenna
           -6 - error in pointing model initialization
            others as defined locally
      (4) = 2HAN for above errors, found in FSERR.CTL
          = 2HST for site defined errors, found in STERR.CTL
      (5) = not used
*/

/* Defined variables */
#define MINMODE 0  /* min,max modes for our operation */
#define MAXMODE 7

#define PORT_NO 1  /* '/dev/lp[012]' to use for parallel port rx control */

/* Include files */
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "antmh.hh"
#include "antmhpar.hh"

extern "C" {
#define class iClass
#include "../../fs/include/params.h" /* FS parameters            */
#include "../../fs/include/fs_types.h" /* FS header files        */
#include "../../fs/include/fscom.h"  /* FS shared mem. structure */
#include "../../fs/include/shm_addr.h" /* FS shared mem. pointer */
#undef class
};

struct fscom *fs;

/* Ordinary C subroutines called */
extern "C" {
void setup_ids();
void putpname(char *);
void skd_wait(char pname[5], long ip[5], unsigned par);
void skd_run(), cls_clr();
int nsem_test();
void logit(char *, int, char *);
int cls_rcv(long iClass, char *buf, int iblen,
	    int *r1, int *r2, int p1, int p2);
void cls_snd(long *iClass, char *buf, int iblen, int p1, int p2);
};


// --- TCP/IP client facilities (from glibc.info):
     
     #include <stdio.h>
     #include <errno.h>
     #include <stdlib.h>
     #include <unistd.h>
     #include <sys/types.h>
     #include <sys/socket.h>
     #include <netinet/in.h>
     #include <netdb.h>
     #include <string.h>  // for 'strerror()'

     #define PORT		5555
     #define SERVERHOST 	"vault.kurp.hut.fi"
     

static void
talkToDammer(const char *hostname,
	     unsigned short int port,
	     const char *sendStr,
	     char *response,
	     unsigned int responseLength)
{
  int sock;
  struct sockaddr_in servername;
     
  /* Create the socket.   */
  sock = socket (PF_INET, SOCK_STREAM, 0);
  if (sock < 0) {
    sprintf(response, "talkToDammer: socket: %s", strerror(errno));
    return;
  }
     
  /* Connect to the server.   */
  {
    struct hostent *hostinfo;
     
    servername.sin_family = AF_INET;
    servername.sin_port = htons (port);
    hostinfo = gethostbyname (hostname);
    if (hostinfo == NULL) {
      sprintf (response, "talkToDammer: host '%s' not found", hostname);
      goto lExit;
    }
    servername.sin_addr = *(struct in_addr *) hostinfo->h_addr;
  }
  if (0 > connect (sock,
     		   (struct sockaddr *) &servername,
     		   sizeof (servername)))
  {
    sprintf(response, "talkToDammer: connect: %s", strerror(errno));
    goto lExit;
  }
     
  /* Send data to the server.   */
  {
    int sendStrLength = strlen(sendStr);
    int nbytes;
    
    nbytes = write (sock, sendStr, sendStrLength);
    if (nbytes < 0) {
      sprintf(response, "talkToDammer: write: %s", strerror(errno));
      goto lExit;
    }
    if (nbytes != sendStrLength) {
      sprintf(response, "talkToDammer: write: wrote only %d bytes of %d",
	      nbytes,
	      sendStrLength);
      goto lExit;
    }

    // Get back a response.
    nbytes = read (sock, response, responseLength-1);
    response[nbytes] = '\0';
  }
 lExit:
  close (sock);
  return;
}  // talkToDammer

// --- end of TCP/IP client facilities


/* antcn main program starts here */
main()
{
  int ierr, nrec, nrecr;
  int dum = 0;
  int r1, r2;
  int imode,i,nchar;
  long ip[5], iClass, clasr;
  char buf[80], buf2[100];

/* Set up IDs for shared memory, then assign the pointer to
   "fs", for readability.
*/

#if 0
  // Debug dammer interface and do nothing else.
  talkToDammer(SERVERHOST,
	       PORT,
	       "FSGET TAMB",
	       buf2,
	       sizeof(buf2));
  printf("dammer response ='%s'\n",
	 buf2);
  exit(EXIT_SUCCESS);
#endif

  setup_ids();
  fs = shm_addr;

/* Put our program name where logit can find it. */

  putpname("antcn");

/* Return to this point to wait until we are called again */

Continue:
  skd_wait("antcn",ip,(unsigned)0);

  imode = ip[0];
  iClass = ip[1];
  nrec = ip[2];

  if (imode < MINMODE || imode > MAXMODE) {
    ierr = -1;
    goto End;
  }

/* Handle each mode in a separate section */

  ierr = 0;
  nrecr = 0;
  clasr = 0;
  switch (imode) {

    case 0:             /* initialize */
#if DEBUG
      strcpy(buf,"Initializing antenna interface");
      logit(buf,0,NULL);
#endif
      {  /* Beware, non-null terminated Hollerith string in fscom: */
        char *end;
        char ant_dev[255];

        end = (char *)memccpy(ant_dev, fs->idevant, ' ', sizeof(ant_dev)-1);
        if (end) {
          *(end-1) = '\0';
        } else {
          ant_dev[sizeof(ant_dev)-1] = '\0';
        }
        fsAntOpen(ant_dev);
      }

      (void)parOpen(PORT_NO);

      fs->ionsor = 0;
      break;

    case 1:             /* source= command */
#if DEBUG
      strcpy(buf,"Commanding to a new source");
      logit(buf,0,NULL);
#endif
/* If 'fs->lsorna' is:
     azel          true corrected coordinates
     azeluncr      encoder units
     xy            encoder units
     stow          Pre-defined stow position.
     service       Pre-defined service position.
     disable       Stops antenna communications. This is useful to stop error
                   messages when here is a problem.
     idle          Monitor the antenna position, but do not command it. This is
                   the normal start-up state for the Field System.
     hold          Maintain the current antenna position.
*/
      if (memcmp(fs->lsorna, "idle    ", 8) == 0) {
        fsGotoIdle();
      } else if (memcmp(fs->lsorna, "hold    ", 8) == 0) {
        fsGotoIdle();
      } else if (
	(memcmp(fs->lsorna, "azel    ", 8) == 0)
	  || (memcmp(fs->lsorna, "azeluncr", 8) == 0)
      ) {
        fsGotoAzEl(fs->ra50, fs->dec50);  // reusing ra=az, dec=el
      } else {
	if (fsStartTracking(fs->lsorna, fs->ra50, fs->dec50, fs->ep1950,
			    fs->radat, fs->decdat, fs->epoch)
	    ) {
	  strcpy(buf,"NAK");
	} else {
	  strcpy(buf,"ACK");
	}
	logit(buf,0,NULL);
      }  /* else start tracking */
      fs->ionsor = 0;
      break;

    case 2:             /* offsets         */
#if DEBUG
      sprintf(buf,"Commanding new offsets: az(rad)==%f, el(rad)==%f", fs->AZOFF, fs->ELOFF);
      logit(buf,0,NULL);
#endif
      fsUpdateOffsets(fs->RAOFF, fs->DECOFF,
		      fs->AZOFF, fs->ELOFF,
		      fs->XOFF, fs->YOFF);
      fs->ionsor = 0;
      break;

    case 3:        /* onsource command with error message */
#if DEBUG
      strcpy(buf,"Checking onsource status, extended error logging");
      logit(buf,0,NULL);
#endif
      fs->ionsor = fsOnsource(buf, NULL);
      logit(buf,0,NULL);
      break;

    case 4:            /* direct antenna= command */
      if (iClass == 0)
        goto End;
      for (i=0; i<nrec; i++) {
        nchar = cls_rcv(iClass,buf,sizeof(buf),&r1,&r2,dum,dum);
        buf[nchar] = '\0';  /* make into a string */
#if DEBUG
        strcpy(buf2,"Received message for antenna: ");
        strcat(buf2,buf);
        logit(buf2,0,NULL);
#endif

	/* Process 'buf' command string. */
	/* 'ANTENNA=PAROUT 2 1' == set bit 2 (0..7) to 1 */
	/* 'ANTENNA=PARIN 2 NORXLOCK RXLOCK' == report 'NORXLOCK' */
	/*          when bit 2 is 0. */
        (void)sscanf(buf, "%s ", buf2);

	// "PAROUT"
	if (memcmp(buf2, "PAROUT", 6) == 0) {
	  int bitNo, newState;

	  (void)sscanf(buf, "%*s %d %d", &bitNo, &newState);
	  parSet(PORT_NO, bitNo, newState);
#if DEBUG
	  sprintf(buf, "Setting parallel bit %d to %d", bitNo, newState);
	  logit(buf,0,NULL);
#endif

	// "PARIN"
	} else if (memcmp(buf2, "PARIN", 5) == 0) {
	  int bitNo, oldState;

	  (void)sscanf(buf, "%*s %d", &bitNo);
	  oldState = parGet(PORT_NO, bitNo);
	  if (oldState) {
	    (void)sscanf(buf, "%*s %*d %*s %s", buf2);
	  } else {
	    (void)sscanf(buf, "%*s %*d %s %*s", buf2);
	  }
#if DEBUG
	  sprintf(buf, "Parallel bit %d is %d == '%s'", bitNo, oldState, buf2);
#else
	  sprintf(buf, "%s", buf2);
#endif
	  logit(buf,0,NULL);

	// "DAMMER"
	} else if (memcmp(buf2, "DAMMER", 6) == 0) {
	  char *sendStr;
	  char response[256];

	  sendStr = &(buf[7]);  // original line after string 'DAMMER '
#if 1
	  sprintf(buf2, "dammer cmd='%s'", sendStr);
	  logit(buf2,0,NULL);
#endif
	  talkToDammer(SERVERHOST, PORT,
		       sendStr, response, sizeof(response));
          // Remove trailing '\n'.
          response[strlen(response)-1] = '\0';
#if DEBUG
	  sprintf(buf, "dammer rsp='%s'", response);
#else
	  // 'buf' is only 80 chars...
	  response[sizeof(buf)-1] = '\0';
	  sprintf(buf, "%s", response);
#endif
	  logit(buf,0,NULL);

	// no 'ANTENNA=command...' matched
	} else {
	  sprintf(buf, "Special antenna command '%s' not implemented.", buf2);
	  logit(buf,0,NULL);
	}

	/* Send successful acknowledge. */
        strcpy(buf,"ACK");
        cls_snd(&clasr,buf,3,dum,dum);
        nrecr += 1;
      }  /* for each comma-separated parameter */
      break;

    case 5:    /* onsource command with no error logging */
#if DEBUG
      strcpy(buf,"Checking onsource status, no error logging");
      logit(buf,0,NULL);
#endif

      // Mask out VAX vlbi_tracker_p slowness to change status
      // and respond immediately after a previous request.
      // (command==5 is the only one FS emits in a loop.)
      usleep(1500000);

      fs->ionsor = fsOnsource(NULL, NULL);
      break;

    case 6:            /* reserved */
      ierr = -1;
      goto End;
      break;

    case 7:    /* onsource command with additional info  */
#if DEBUG
      strcpy(buf,"Checking onsource status, log tracking data");
      logit(buf,0,NULL);
#endif
      fs->ionsor = fsOnsource(buf, buf2);
      logit(buf,0,NULL);
      logit(buf2,0,NULL);
/* A sample response from 'track' SNAP command help file:
tr   yy/ddd.hh:mm:ss       cmd1    cmd2    delta1    delta2
pr                         prd1    prd2    praz      prel
co                         cor1    cor2    eqeq      refr
 */
  }  /* end of switch */

End:
  ip[0] = clasr;
  ip[1] = nrecr;
  ip[2] = ierr;
  memcpy(ip+3,"AN",2);
  ip[4] = 0;
  goto Continue;

}
