// antmhpar.c -- Metsahovi parallel receiver interface, for FS9.
// Copyright (C) 1995 Ari Mujunen. (Ari.Mujunen@hut.fi)

// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License.
// See the file 'COPYING' for details.

// $Log: antmhpar.cc,v $
// Revision 1.1  1997/03/13  12:30:54  prog
// Initial revision
//


#include "registration.h"
RCSID(antmhpar_cc, "$Id: antmhpar.cc,v 1.1 1997/03/13 12:30:54 prog Exp $");

#include "antmhpar.hh"


/*
 *  AUTHOR: Sven Goldt (goldt@math.tu-berlin.de)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
*/
/*
 *
 * Changes in 1.2: amn@bigbang.nfra.nl removed "oldlp.h", converted
                   to "new" LPGETIRQ ioctl(), 
 * Changes in 1.1: Fixed bit meaning for low active lines ! 
 *
*/

#include <stdio.h>
#include <unistd.h>  /* ioperm() */
#include <linux/lp.h>
/* DAMNED ! - Someone crippled that file !!! */
/* amn: Yes, yes, data encapsulation, you know... :-) */
/*      That table is currently hidden inside the kernel. */
/*      For our direct parallel I/O port references, here's a copy */
/*      (from '/usr/src/linux/drivers/char/lp.c'): */
struct lp_struct lp_table[] = {
	{ 0x3bc, 0, 0, LP_INIT_CHAR, LP_INIT_TIME, LP_INIT_WAIT, NULL, NULL, },
	{ 0x378, 0, 0, LP_INIT_CHAR, LP_INIT_TIME, LP_INIT_WAIT, NULL, NULL, },
	{ 0x278, 0, 0, LP_INIT_CHAR, LP_INIT_TIME, LP_INIT_WAIT, NULL, NULL, },
}; 
#define LP_NO 3

/* The direct I/O port reference macros reside here. */
/* They work _only_ if the source files using them are compiled */
/* with '-O' optimization flag on; they rely on 'inline' code generation. */
#include <asm/io.h>


static unsigned char vCurrentPortByte[3];


/* Get port access to parallel port /dev/lp[012], return 0 if successful. */
int parOpen(int port)
{
  /***fprintf(stderr, "I/O base address is 0x%x\n",LP_B(port));***/

  /* Getting the permission to directly access a given range */
  /* of I/O port addresses (below 0x3ff, above that see 'man iopl'). */
  if (ioperm(
	/*from address=>*/         LP_B(port),
	/*number of addresses=>*/  2,
	/*turn on=>*/              1  /* 0 to turn off */
  ) < 0) {
    fprintf(stderr, "antmhpar.cc: access to parallel port 0x%x denied\n", LP_B(port));
    return 1;
  }
  vCurrentPortByte[port] = 0xFF;
  outb(vCurrentPortByte[port], LP_B(port));
  return 0;
}  /* parOpen */

/* Return the value 0/1 of BIT of parallel PORT status register. */
int parGet(int port, int bit) {
  unsigned char status;

  /* Read the status byte directly from 8255. */
  status = inb(LP_B(port)+1);
  return ( (status & (1 << bit)) ? 1 : 0 );
}  /* parGet */

/* Set BIT of parallel PORT to 0 if VAL == 0, else to 1. */
void parSet(int port, int bit, int val) {
  unsigned char value;

  value = vCurrentPortByte[port];
  if (val) {
    value |= (1 << bit);
  } else {
    value &= ~(1 << bit);
  }
  /* Send out desired value to the parallel port outgoing data lines: */
  outb(value, LP_B(port));
  vCurrentPortByte[port] = value;
}  /* parSet */

#if 0
Just repeating stuff from <linux/lp.h>, for docum. purposes:
(Bit assignments of input register / Centronics input pins.)
    LP_PBUSY	0x80  /* inverted input, active high */
    LP_PACK	0x40  /* unchanged input, active low */
    LP_POUTPA	0x20  /* unchanged input, active high */
    LP_PSELECD	0x10  /* unchanged input, active high */
    LP_PERRORP	0x08  /* unchanged input, active low */

And from Darfs 'site_io.pas':
  cRXNoiseDiodeBit=0;  (* port data register, bit 0 *)
  (* cRXHighNoiseDiodeBit=1; is reserved for a second noise diode in 22GHz *)
  cRXHotAbsorberPlateBit=2;  (* port data register, bit 2 *)
  (* All other bits are reserved and set to 1 (TTL high)... *)
  cRXAlwaysHighBit=7;  (* ... and bit 7 is guaranteed to stay that way. *)

  (* Both noise diodes and the hot plate are on when bit is =0. *)
  (* Off when bit =1. *)


  cRXLockedBit=7;  (* port status register, Centronics pin 11, Busy *)

#endif
